package com.hyperling.examplegame_practice;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;

import java.util.ArrayList;

/**
 * Created by usb on 11/9/14.
 */
public class GameGrid {
    GameView gv;
    Paint paint = new Paint();

    static int ROWS;
    static int COLUMNS;

    static GridCell[][] CELL;

    public GridCell startCell;
    public GridCell endCell;

    public GridCell basicUpgradeCell = new GridCell();
    public GridCell slowUpgradeCell = new GridCell();

    ArrayList<GridCell> buildCells = new ArrayList<GridCell>();
    ArrayList<GridCell> towers = new ArrayList<GridCell>();

    //ArrayList<GridCell> pauseButton = new ArrayList<GridCell>();

    static int CELL_COUNT;

    static int CELL_WIDTH;
    static int CELL_HEIGHT;

    boolean showUpgrades;

    public GameGrid(int numRows, int numColumns, GameView gameView){
        paint.setStrokeWidth(0);

        ROWS = numRows;
        COLUMNS = numColumns;

        CELL = new GridCell[ROWS][COLUMNS];

        gv = gameView;

        CELL_COUNT = ROWS*COLUMNS;

        CELL_WIDTH = gv.getWidth()/COLUMNS;
        CELL_HEIGHT = gv.getHeight()/ROWS;

        int q = 0;
        for(int i = 0; i < ROWS; i++){
            for(int j = 0; j < COLUMNS; j++){
                int[] id = {j,i}; // Need to be switched for some reason
                CELL[i][j] = new GridCell(id, CELL_WIDTH, CELL_HEIGHT);

                ++q;
                if(q == CELL_COUNT){
                    System.out.println("All cells created!");
                }
            }
        }
    }

    public void onDraw(Canvas canvas){

        paintCells(canvas);

        paintBuildableBorders(canvas);

        paintTowers(canvas);

        paintTowerInfo(canvas);

        if(showUpgrades){
            paintUpgradeChoices(canvas);
        }
        else{
            basicUpgradeCell.color = new GridCell().color;
            slowUpgradeCell.color = new GridCell().color;

            basicUpgradeCell = new GridCell();
            slowUpgradeCell = new GridCell();
        }

    }

    private void paintCells(Canvas canvas){
        for(int i = 0; i < ROWS; i++){
            for(int j = 0; j < COLUMNS; j++){

                paint.setColor(CELL[i][j].getColor());

                int side[] = CELL[i][j].getSides();

                if (side[0] < 0) {
                    side[0] = 0;
                }
                if (side[1] < 0) {
                    side[1] = 0;
                }
                if (side[2] > gv.getWidth()) {
                    side[2] = gv.getWidth();
                }
                if (side[3] > gv.getHeight()) {
                    side[3] = gv.getHeight();
                }

                canvas.drawRect(side[0], side[1], side[2], side[3], paint);
                //} catch(Exception e){
                //System.out.println("GameGrid.paintCells() threw an error at cell[" + i + "][" + j + "]: " + e.toString());
                //System.out.println("(int)Color is " + tempC);
                //}

            }
        }
    }

    public void paintBuildableBorders(Canvas canvas){
        for(GridCell buildCell : buildCells){

            paint.setColor(buildCell.buildableInterior);

            /*
            int[] temp = buildCell.id;
            int i = temp[0];
            int j = temp[1];
            */

            int[]side = buildCell.side;

            int xSpace = buildCell.width/10;
            int ySpace = buildCell.height/10;

            canvas.drawRect(side[0] + xSpace, side[1] + ySpace, side[2] - xSpace, side[3] - ySpace, paint);
        }
    }

    public void paintTowers(Canvas canvas){
        String slow = "";
        for(GridCell tower : towers){

            paint.setColor(tower.innerColor);

            /*
            int[] temp = tower.id;
            int i = temp[0];
            int j = temp[1];
            */
            int[]side = tower.side;

            int xSpace = tower.width/4;
            int ySpace = tower.height/4;

            canvas.drawRect(side[0] + xSpace, side[1] + ySpace, side[2] - xSpace, side[3] - ySpace, paint);

            paint.setColor(tower.buildableInterior);

            if(tower.isSlowTower){
                slow = "S";
            } else {
                slow = "";
            }

            canvas.drawText(tower.towerType + slow, tower.centerForTextX, tower.centerForTextY, paint);
        }
    }

    public void paintTowerInfo(Canvas canvas){
        GridCell fakeTower = new GridCell();
        fakeTower.setTower();

        int firstcost = 2;
        int cost = firstcost;
        int upgradecost;
        int spent;

        GridCell printCell = CELL[ROWS-2][0];

        paint.setColor(Color.YELLOW);
        canvas.drawText("Tower build costs:", printCell.centerForTextX, printCell.centerForTextY, paint);

        for(int i = 0; i < COLUMNS/2; i++){
            printCell = CELL[ROWS-1][i];
            printCell.color = Color.WHITE;

            upgradecost = fakeTower.towerUpgradeCost;

            paint.setColor(fakeTower.innerColor);
            canvas.drawText("" + cost, printCell.centerForTextX, printCell.centerForTextY, paint);

            fakeTower.upgrade();

            cost = upgradecost;
        }
    }

    public void paintUpgradeChoices(Canvas canvas){
        GridCell printCell = CELL[ROWS-2][COLUMNS-4];

        paint.setColor(Color.YELLOW);
        canvas.drawText("Upgrade choices:", printCell.centerForTextX, printCell.centerForTextY, paint);

        int choice = 0;
        for(int i = COLUMNS-1; i >= COLUMNS - printCell.towerChoices; i--){
            printCell = CELL[ROWS-1][i];

            if(choice == 0){
                printCell.color = Color.RED;
                canvas.drawText("Basic", printCell.centerForTextX, printCell.centerForTextY, paint);
                basicUpgradeCell = printCell;
            }
            else if(choice == 1){
                printCell.color = Color.BLUE;
                canvas.drawText("Slow", printCell.centerForTextX, printCell.centerForTextY, paint);
                slowUpgradeCell = printCell;
            }

            choice = choice + 1;
        }

    }

    public void setStartCell(int[] id){
        CELL[id[0]][id[1]].setStart();
        startCell = CELL[id[0]][id[1]];
    }

    public void setEndCell(int[] id){

        CELL[id[0]][id[1]].setEnd();
        endCell = CELL[id[0]][id[1]];
    }

    public void setPathCell(int[] id){
        CELL[id[0]][id[1]].setPath();
    }

    public void setBuildableCell(int[] id){
        CELL[id[0]][id[1]].setBuildable();
        buildCells.add(CELL[id[0]][id[1]]);
    }

    public void setTowerCell(int[] id){
        CELL[id[0]][id[1]].setTower();
        towers.add(CELL[id[0]][id[1]]);
        //System.out.println("Tower added: " + id[0] + ", " + id[1]);
    }

    /*
    public void setPausableCell(int[] id){
        pauseButton.add(CELL[id[0]][id[1]]);
    }
    */

    public void setNullCell(int[] id){
        CELL[id[0]][id[1]].setNull();
    }
}
