package com.hyperling.examplegame_practice;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Looper;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

/**
 * Created by usb on 10/16/14.
 *
 *
 */
public class GameView extends SurfaceView implements Serializable {

    private SurfaceHolder holder;

    private ArrayList<Minion> minions = new ArrayList<Minion>();
    private ArrayList<Bullet> bullets = new ArrayList<Bullet>();
    //private int [][] minionTypes = new int [3][20]; // {amount, drawable, role}

    private GameLoopThread gameLoopThread;
    private Context baseContext;

    Paint paint = new Paint();


    Timer timer = new Timer();
    TimerTask startGameTask;
    TimerTask startNextLevelTask;


    GameGrid gameGrid;

    // Map specs
    int numRows = 9;
    int numColumns = 16;
    int[] startCoords = {4, 0};
    int[] endCoords = {4, 15};
    int[] pathCoordsX = {4,4,4,4,4,4,4,4,4,4 ,4 ,4, 4, 4 ,-666}; // Row
    int[] pathCoordsY = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,-666};// Column
    int[] buildableCoordsX = {5,5,5,5,5,5,5,5,5, 5, 5, 5, 3,3,3,3,3,3,3,3,3, 3, 3, 3, -666};
    int[] buildableCoordsY = {2,3,4,5,6,7,8,9,10,11,12,13,2,3,4,5,6,7,8,9,10,11,12,13,-666};
    /*
    int[] pausableCoordsX = { 1, 1, 1, 1, 2, 2, 2, 2,-666};
    int[] pausableCoordsY = {11,12,13,14,11,12,13,14,-666};
    */

    int startX, startY, endX, endY;
    int[] startSides, endSides;

    int defaultLives = 10;
    int lives = defaultLives;

    //boolean spawn = true;
    int defaultExtraSpawn = 3;
    int spawnNum = defaultExtraSpawn;
    int spawnMore = spawnNum;
    int spawnMoreBosses = 0;

    int defaultRole = 1;
    int role = defaultRole;

    int defaultSpeed = 250;
    int speed = defaultSpeed;

    boolean remove;
    int livesLost;

    GridCell selectedTower;

    boolean gridCreated = false;

    boolean shoot;
    int shootMinion;
    int shootTower;

    boolean gameOver = false;

    boolean paused = false;
    private Boolean mainPause = false;
    public long mainPauseTime = 0;
    public long mainUnPauseTime = 0;
    private int minionPauseSpeed = 0;
    int startTime = 2000;

    int round = 1;
    int wave = 0;

    int killCount = 0;

    int defaultMoney = 4;
    int money = defaultMoney;

    private long lastClick;

    @SuppressLint("WrongCall")
    public GameView(Context context) {

        super(context);

        baseContext = context;

        gameLoopThread = new GameLoopThread(this);

        holder = getHolder();

        holder.addCallback(new SurfaceHolder.Callback() {

            @Override
            public void surfaceDestroyed(SurfaceHolder holder) {

                boolean retry = true;

                gameLoopThread.setRunning(false);

                while (retry) {

                    try {

                        gameLoopThread.join();

                        retry = false;

                    } catch (InterruptedException e) {

                    }

                    gameLoopThread = new GameLoopThread(GameView.this);
                }
            }

            @Override
            public void surfaceCreated(SurfaceHolder holder) {
                /*
                Canvas c = holder.lockCanvas(null);
                onDraw(c);
                holder.unlockCanvasAndPost(c);
                 */
                paint.setColor(Color.YELLOW);
                paint.setStrokeWidth(0);

                /*
                timer = new Timer();
                startGameTask = new TimerTask() {
                    @Override
                    public void run() {
                        createMinions();

                        gameLoopThread.setRunning(true);

                        gameLoopThread.start();
                    }
                };

                startNextLevelTask = new TimerTask() {
                    @Override
                    public void run() {
                        createMinions();
                    }
                };
                */

                createGrid();

                paused = true;

                startGameTask = new TimerTask() {
                    @Override
                    public void run() {
                        paused = false;

                        if(wave == 0){
                            startNextLevel();
                        }

                        startTime = 0;
                    }
                };

                gameLoopThread.setRunning(true);

                gameLoopThread.start();

                //System.out.println("Game will start in 5 seconds...");
                timer.schedule(startGameTask, startTime);
            }

            @Override
            public void surfaceChanged(SurfaceHolder holder, int format,
                                       int width, int height) {
            }

        });

    }


    @Override
    protected void onDraw(Canvas canvas) {
        try {
            canvas.drawColor(Color.BLACK);

            // Display stats
            canvas.drawText("Level: Round " + round + " wave " + wave, gameGrid.CELL[0][0].x, gameGrid.CELL[0][0].centerY, paint);
            canvas.drawText("Money: " + money, gameGrid.CELL[1][0].x, gameGrid.CELL[1][0].y, paint);
            canvas.drawText("Lives left: " + lives, gameGrid.CELL[1][0].x, gameGrid.CELL[1][0].centerY, paint);

            if (!gameOver) {

                gameGrid.onDraw(canvas);

                shootMinion = 0;

                for (GridCell tower : gameGrid.towers) {
                    tower.isReloading(mainPauseTime, mainUnPauseTime);
                    gameGrid.paintTowers(canvas);
                }

                if (mainPause) {
                    for (Minion minion : minions) {
                        if (minion.xSpeed > 0) {
                            minionPauseSpeed = minion.xSpeed;
                            minion.xSpeed = 0;
                        }
                        minion.onDraw(canvas);
                    }
                    for(Bullet bullet : bullets){
                        if(bullet.pauseSpeed == 1){
                            bullet.pauseSpeed = 0;
                        }
                        bullet.onDraw(canvas);
                    }
                } else {
                    for (Minion minion : minions) {
                        if (minion.xSpeed == 0) {
                            minion.xSpeed = minionPauseSpeed;
                        }
                    }

                    for (Bullet bullet : bullets){
                        if(bullet.pauseSpeed == 0){
                            bullet.pauseSpeed = 1;
                        }
                    }

                    if (!paused) {
            /*
            // Paint the track!
            paintSquare(canvas, startX, startY, Color.GREEN);
            paintSquare(canvas, endX, endY, Color.RED);
            paintRect(canvas, startX + trackSize / 2, startY + trackSize / 2, endX - startX - trackSize, trackSize, Color.YELLOW);
            */

                        if (wave == 6) {
                            //Spawn more bosses
                            if (minions.size() == 0 && spawnMoreBosses > 0) {
                                createBosses();
                                --spawnMoreBosses;
                            }
                        } else {
                            // Check if able to spawn
                            if (minions.size() == 0 && spawnMore > 0) {
                                createMinions();
                                --spawnMore;
                            }
                        }

                        // Move the bullets
                        for(Bullet bullet : bullets){
                            bullet.onDraw(canvas);
                            System.out.println("Drawing bullet for minion " + minions.indexOf(bullet.minion));

                        }

                        // See if bullet is hitting minion
                        /*
                        for(Bullet bullet : bullets){
                            if (!bullet.spent) {
                                if (bullet.minion.health > 0 && bullet.minion.isCollision(bullet.centerX, bullet.centerY)) {
                                    damageMinion(bullet.minion, bullet.damage);
                                    bullet.spent = true;
                                }
                            }
                        }
                        */
                        for(Bullet bullet : bullets){
                            if (!bullet.spent) {
                                if (bullet.minion.health > 0 && bullet.isCollision()) {
                                    damageMinion(bullet.minion, bullet.damage);
                                    bullet.spent = true;
                                }
                            }
                        }

                        //for (int n = minions.size() - 1; n >= 0; n--) {
                        for (Minion minion : minions) {
                            //Minion minion = minions.get(n);

                            minion.onDraw(canvas);



                /*
                // Check if able to spawn
                if(minion.isCollision(startSides)){
                    spawn = false;
                } else {
                    spawn = true;
                }
                */

                            if (wave == 6) {
                                //Spawn more bosses
                                if ((!minions.get(minions.size() - 1).isCollision(startSides)) && spawnMoreBosses > 0) {
                                    createBosses();
                                    --spawnMoreBosses;
                                }
                            } else {
                                // Check if able to spawn
                                if ((!minions.get(minions.size() - 1).isCollision(startSides)) && spawnMore > 0) {
                                    createMinions();
                                    --spawnMore;
                                }
                            }



                /*
                // Check if under tower fire
                int[] minionCell = minion.getCellLocation(numRows, numColumns);
                int i = minionCell[0];
                int j = minionCell[1];
                GridCell testTower;

                // Test left cell
                if(i > 1) {
                    testTower = gameGrid.CELL[i - 1][j];
                    if (testTower.isTower && System.currentTimeMillis() - testTower.lastShot > 500) {
                        shootMinion(testTower, minion);
                        break;
                    }
                }
                // Test top cell
                if(j > 1) {
                    testTower = gameGrid.CELL[i][j - 1];
                    if (testTower.isTower && System.currentTimeMillis() - testTower.lastShot > 500) {
                        shootMinion(testTower, minion);
                        break;
                    }
                }
                    // Test right cell
                if(i < numRows) {
                    testTower = gameGrid.CELL[i + 1][j];
                    if (testTower.isTower && System.currentTimeMillis() - testTower.lastShot > 500) {
                        shootMinion(testTower, minion);
                        break;
                    }
                }
                    // Test bottom cell
                if(j < numColumns) {
                    testTower = gameGrid.CELL[i][j + 1];
                    if (testTower.isTower && System.currentTimeMillis() - testTower.lastShot > 500) {
                        shootMinion(testTower, minion);
                        break;
                    }
                }
                */
                /*
                for(GridCell towerCell: gameGrid.towers){
                    shoot = !towerCell.isReloading();
                    //System.out.println("Tower " + towerCell.id[0] + ", " + towerCell.id[1] + "is in range: " + towerCell.isInRange(minion.x, minion.y));
                    if(shoot) {
                        if(towerCell.isInRange(minion.x, minion.y)){
                            shootMinion(towerCell, minion);
                            break;
                        }
                    }
                }
                */

                            for (GridCell towerCell : gameGrid.towers) {
                                //System.out.println("Tower " + towerCell.id[0] + ", " + towerCell.id[1] + "is in range: " + towerCell.isInRange(minion.x, minion.y));
                                if (!towerCell.isReloading(mainPauseTime, mainUnPauseTime)) {
                                    if (towerCell.isInRange(minion.x, minion.y)) {
                                        if (minion.x >= minions.get(shootMinion).x) {
                                            shoot = true;
                                            shootMinion = minions.indexOf(minion);
                                            shootTower = gameGrid.towers.indexOf(towerCell);
                                            //shootMinion(towerCell, minion);
                                            //break;
                                        } else {
                                            shoot = true;
                                        }
                                    }
                                }
                            }

                /*
                // Check if minion has reached end
                if(minion.isCollision(endSides)){
                    --lives;
                    remove = true;
                }
                */

                            // Check if minion has reached end
                            if (minion.isCollision(endSides)) {
                                remove = true;
                                livesLost = 1;
                            }
                        }

                        for(Bullet bullet : bullets){
                            if(bullet.spent){
                                bullets.remove(bullet);
                            }
                        }

                        if (shoot) {
                            if (minions.get(shootMinion).health <= gameGrid.towers.get(shootTower).damage) { // && shootMinion == 0) {
                                remove = false;
                            }
                            shootMinion(gameGrid.towers.get(shootTower), minions.get(shootMinion));
                            shoot = false;
                        }


                        // Remove minion if needed
                        if (remove) {
                            for(Bullet bullet : bullets){
                                if(minions.indexOf(bullet.minion) == 0){
                                    bullet.spent = true;
                                }
                            }
                            minions.remove(minions.get(0));
                            lives = lives - livesLost;
                            //money = money + livesLost;
                            remove = false;
                        }

            /*
            // Spawn more if needed
            if(spawnMore > 0 && spawn){
                createMinions();
                --spawnMore;
            }

            // Remove minion if needed
            if(remove){
                minions.remove(minions.get(0));
                remove = false;
            }
            */

                        // If minions have been defeated, advance levels!
                        if (spawnMore == 0 && spawnMoreBosses == 0 && minions.size() == 0) {
                            startNextLevel();
                        }

                        // If all lives have been lost, it's game over!
                        if (lives <= 0) {
                            gameOver = true;
                        }

            /*
            // Update the runners
            try {
                for (Minion minion : minions) {
                    minion.onDraw(canvas);
                    canvas.drawText("" + minion.health, minion.x + minion.width / 8, minion.y - trackSize / 2, paint);

                    //Spawn minion after last leaves spawn
                    if (minion.x > startX + trackSize / 2 && spawnMore > 0) {
                        spawn = true;
                    }
                    else {
                        spawn = false;
                    }

                    // If a minion reaches the end, remove him and a life
                    if (minion.isCollision(endX, endY)) {
                        minions.remove(minion);
                        --lives;

                        // If out of lives, game over!
                        if (lives <= 0) {
                            gameOver = true;
                        }
                    }

                }
            }
            catch(Exception e){
                System.out.println(e.toString());
            }

            if(spawn){
                minions.add(createMinion(R.drawable.sprite_green01, 5));
                --spawnMore;
                spawn = false;
            }
            */
                    }
                }
            } else {
                canvas.drawText("GAME OVER", getWidth() / 2, getHeight() / 2, paint);
                canvas.drawText("Round " + round + " wave " + wave, getWidth() / 2, getHeight() / 2 + 20, paint);
            }
        }
        catch(Exception e){
            System.out.println("Error in onDraw!");
            e.printStackTrace();
            System.out.println("Error in onDraw!");
        }
    }

    private void createBosses(){
        minions.add(createMinion(R.drawable.demon_king, round*10, speed*2));

    }

    private void createMinions(){
        //bmp = BitmapFactory.decodeResource(getResources(), R.drawable.sprite_green01);
        /* */

        minions.add(createMinion(R.drawable.sprite_green01, role, speed));

        // */
        /*
        for(; minionTypes[0][level] >= 0; --minionTypes[0][level]){
            minions.add(createMinion(minionTypes[1][level], minionTypes[2][level]));
        }
        // */
    }

    private Minion createMinion(int resource, int role, int speed) {

        Bitmap bmp = BitmapFactory.decodeResource(getResources(), resource);

        return new Minion(this, bmp, role, speed, startX, startY);

    }

    private Bullet createBullet(int bulletresource, int damage, int startX, int startY, Minion minion){

        Bitmap bulletbmp = BitmapFactory.decodeResource(getResources(), bulletresource);

        return new Bullet(this, bulletbmp, damage, startX, startY, minion);
    }

    private void createGrid(){

        if(!gridCreated) {
            gameGrid = new GameGrid(numRows, numColumns, GameView.this);

            gameGrid.setStartCell(startCoords);
            gameGrid.setEndCell(endCoords);

            for (int i = 0; true; i++) {
                if (pathCoordsX[i] == -666) {
                    break;
                }
                int[] tempID = {pathCoordsX[i], pathCoordsY[i]};
                gameGrid.setPathCell(tempID);
            }

            for (int i = 0; true; i++) {
                if (buildableCoordsX[i] == -666) {
                    break;
                }
                int[] tempID = {buildableCoordsX[i], buildableCoordsY[i]};
                gameGrid.setBuildableCell(tempID);
            }

        /*
        for(int i = 0; true; i++){
            if(pausableCoordsX[i] == -666){
                break;
            }
            int[] tempID = {pausableCoordsX[i], pausableCoordsY[i]};
            gameGrid.setPausableCell(tempID);
        }
        */

            startX = gameGrid.startCell.centerX;
            startY = gameGrid.startCell.centerY;
            startSides = gameGrid.startCell.getSides();

            endX = gameGrid.endCell.centerX;
            endY = gameGrid.endCell.centerY;
            endSides = gameGrid.endCell.getSides();

            gridCreated = true;
        }
    }
/*
    private void paintSquare(Canvas canvas, int x, int y, int color){
        paint.setColor(color);
        paint.setStrokeWidth(0);
        int side[] = new int[4];

        side[0] = x - trackSize/2;//left
        side[1] = y + trackSize/2;//top
        side[2] = x + trackSize/2;//right
        side[3] = y - trackSize/2;//bottom

        if(side[0] < 0) {
            side[0] = 0;
        }
        if(side[1] < 0) {
            side[1] = 0;
        }
        if(side[2] > getWidth()) {
            side[2] = getWidth();
        }
        if(side[3] > getHeight()) {
            side[3] = getHeight();
        }

        canvas.drawRect(side[0], side[1], side[2], side[3], paint);
    }

    private void paintRect(Canvas canvas, int x, int y, int width, int height, int color){
        paint.setColor(color);
        paint.setStrokeWidth(0);
        int side[] = new int[4];

        side[0] = x;//left
        side[1] = y;//top
        side[2] = x + width;//right
        side[3] = y - height;//bottom

        if(side[0] < 0) {
            side[0] = 0;
        }
        if(side[1] < 0) {
            side[1] = 0;
        }
        if(side[2] > getWidth()) {
            side[2] = getWidth();
        }
        if(side[3] > getHeight()) {
            side[3] = getHeight();
        }

        canvas.drawRect(side[0], side[1], side[2], side[3], paint);
    }
*/

    @Override
    public boolean onTouchEvent(MotionEvent event){

        if (System.currentTimeMillis() - lastClick > 250) {

            lastClick = System.currentTimeMillis();

            float x = event.getX();

            float y = event.getY();

            synchronized (getHolder()) {

                /*
                for (int i = minions.size() - 1; i >= 0; i--) {

                    Minion minion = minions.get(i);

                    if (minion.isCollision(x, y)) {

                        damageMinion(minion);

                        if (minions.size() == 0) {
                            startNextLevel();
                        }
                        break;

                    }

                }
                */
                /*
                for(GridCell pause : gameGrid.pauseButton){
                    if(pause.isCollision(x,y) && !paused){
                        paused = true;
                        pauseTime = System.currentTimeMillis();
                    }
                    else if(pause.isCollision(x,y) && paused){
                        paused = false;
                        unPauseTime = System.currentTimeMillis();
                    }
                }
                */

                for(GridCell tower: gameGrid.towers){
                    if(tower.isCollision(x,y)){ //) && money >= tower.towerUpgradeCost){
                        /*
                        money = money - tower.towerUpgradeCost;
                        tower.upgrade();
                        */
                        gameGrid.showUpgrades = true;
                        selectedTower = tower;
                    }
                }

                if(gameGrid.showUpgrades){

                    if(!selectedTower.isSlowTower){
                        if(gameGrid.basicUpgradeCell.isCollision(x,y) && money >= selectedTower.towerUpgradeCost){
                            money = money - selectedTower.towerUpgradeCost;
                            selectedTower.upgrade();
                            gameGrid.showUpgrades = false;
                        }
                        else if(gameGrid.slowUpgradeCell.isCollision(x,y)){
                            selectedTower.setSlowTower();
                            gameGrid.showUpgrades = false;
                        }
                    }
                    else if(selectedTower.isSlowTower){
                        if(gameGrid.basicUpgradeCell.isCollision(x,y)){
                            selectedTower.setBasicTower();
                            gameGrid.showUpgrades = false;
                        }
                        else if(gameGrid.slowUpgradeCell.isCollision(x,y) && money >= selectedTower.towerUpgradeCost){
                            money = money - selectedTower.towerUpgradeCost;
                            selectedTower.upgrade();
                            gameGrid.showUpgrades = false;
                        }
                    }
                }

                for(int i = 0; true; i++){
                    if(buildableCoordsX[i] == -666){
                        break;
                    }
                    if(gameGrid.CELL[buildableCoordsX[i]][buildableCoordsY[i]].isCollision(x,y) && !gameGrid.CELL[buildableCoordsX[i]][buildableCoordsY[i]].isTower){
                        if(money >= gameGrid.CELL[buildableCoordsX[i]][buildableCoordsY[i]].defaultTowerCost) {
                            int[] temp = {buildableCoordsX[i], buildableCoordsY[i]};
                            gameGrid.setTowerCell(temp);
                            money = money - gameGrid.CELL[buildableCoordsX[i]][buildableCoordsY[i]].defaultTowerCost;
                        }
                    }
                }

            }

        }

        return true;
    }

    private void startNextLevel(){
        //long pauseTime = System.currentTimeMillis();
        // Pause for 5 seconds
        //while(System.currentTimeMillis() - pauseTime < 5000){};

        /*
        while(killCount >= 5){
            money = money + round;
            killCount = killCount - 5;
        }
        */
        //money = money + round;

        ++wave;

        if(wave > 6){
            //money = money + round;

            ++round;
            wave = 1;
            ++lives;

            defaultSpeed = defaultSpeed - 5;

            role = role + round-1;

            money = money + role;

            spawnMore = spawnNum - spawnNum/3;

            //spawnNum = defaultExtraSpawn + (round-1);
        }

        ++spawnNum;

        speed = defaultSpeed - wave*wave;

        startNextLevelTask = new TimerTask() {
            @Override
            public void run() {
                paused = false;
                if (wave == 6){
                    spawnMoreBosses = round -1;
                    createBosses();
                } else {
                    spawnMore = spawnNum;
                    createMinions();
                }
            }
        };
        paused = true;
        timer.schedule(startNextLevelTask, 3000);
    }

    /*
    private void startNewGame(){
        AlertDialog.Builder builder = new AlertDialog.Builder(baseContext);
        builder.setMessage("Game over, try again?");

        builder.setNegativeButton("No", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                android.os.Process.killProcess(android.os.Process.myPid());

                dialogInterface.cancel();
            }
        });

        builder.setPositiveButton("Yes", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                lives = defaultLives;
                level = 0;
                gameGrid = new GameGrid(numRows, numColumns, GameView.this);

                createMinions();

                dialogInterface.cancel();
            }
        });

        AlertDialog alertDialog = builder.create();
        alertDialog.show();
    }
    */

    public void shootMinion(GridCell tower, Minion minion){
        tower.lastShot = System.currentTimeMillis();
        tower.isReloading(mainPauseTime, mainUnPauseTime);

        if(tower.isSlowTower){
            slowMinion(minion);
        }

        bullets.add(createBullet(R.drawable.bullet_cc2, tower.damage, tower.centerX, tower.centerY, minion));

        // Now called on collision with bullet
        //damageMinion(minion, tower.damage);
    }

    public void damageMinion(Minion minion, int damage){
        minion.health = minion.health - damage;

        if(minion.health <= 0) {
            for(Bullet bullet : bullets){
                if(bullet.minion.health <= 0){
                    bullets.remove(bullet);
                }
            }
            System.out.println("Dying minion health = " + minion.health);
            minions.remove(minion);
            ++killCount;

            // Award for kill count
            if(wave == 6){
                ++money;
            } else {
                while (killCount >= 5) {
                    ++money;
                    killCount = killCount - 5;
                }
            }
        }
    }

    public void slowMinion(Minion minion){
        minion.slowedTime = System.currentTimeMillis();
        minion.isSlowed = true;
    }

    public void setMainPause(Boolean pause){
        mainPause = pause;

        if(mainPause){
            mainPauseTime = System.currentTimeMillis();
        } else {
            mainUnPauseTime = System.currentTimeMillis();
        }
    }
    /*

    public void stopThreads(){
        gameLoopThread.setRunning(false);
        try {
            //gameLoopThread.join();
            gameLoopThread.suspend();
        } catch (Exception e) {
            System.out.println(e.toString());
            e.printStackTrace();
        }
    }

    public void startThreads(){
        gameLoopThread.setRunning(true);

        //gameLoopThread.start();
    }
    */

    /*
    public void loadData(){

    }

    public String saveData(){
        String save;

        save = "";

        return save;
    }
    */
}
