package com.hyperling.examplegame_practice;

import android.graphics.Color;
/**
 * Created by usb on 11/9/14.
 */
public class GridCell {
    int[] id; // x,y == row,column
    int x,y; // coordinates

    int height;
    int width;

    int side[] = new int[4]; // Left, Top, Right, Bottom

    int centerX; // center coordinates
    int centerY; // center coordinates
    int centerForTextX;
    int centerForTextY;

    int startC = Color.GREEN;
    int endC = Color.RED;
    int pathC = Color.YELLOW;
    int buildableC = Color.DKGRAY;
    int buildableInterior = Color.CYAN;
    int towerC = Color.BLUE;
    int reloadingC = Color.WHITE;
    int nullC = Color.TRANSPARENT;

    int color = nullC;
    int innerColor = color;

    int towerType = 1;

    int defaultTowerCost = 2;
    int towerCost = defaultTowerCost;
    int towerUpgradeCost;
    int towerChoices = 2;

    boolean isStart;
    boolean isEnd;
    boolean isPath;
    boolean isBuildable;
    boolean isTower;
    boolean isSlowTower = false;
    private boolean isReloading = false;
    boolean isNull = true;

    long lastShot = 0;
    int defaultReload = 2500;
    int reload = defaultReload;

    int damage = 1;

    public GridCell(){
        findTowerCost();
    }

    public GridCell(int[] id, int width, int height){
        this.id = id;

        x = id[0] * width;
        y = id[1] * height;

        this.height = height;
        this.width = width;

        side[0] = x;//left
        side[1] = y;//top
        side[2] = x + width;//right
        side[3] = y + height;//bottom

        centerX = (side[0]+side[2])/2;
        centerY = (side[1]+side[3])/2;

        centerForTextX = centerX - width/24;
        centerForTextY = centerY + height/16;

        findTowerCost();
        /*
        if(side[0] < 0) {
            side[0] = 0;
        }
        if(side[1] < 0) {
            side[1] = 0;
        }
        if(side[2] > getWidth()) {
            side[2] = getWidth();
        }
        if(side[3] > getHeight()) {
            side[3] = getHeight();
        }
        */
    }

    public int getColor(){
        return color;
    }

    public int[] getSides(){
        return side;
    }

    private void setAllFalse(){
        isStart = false;
        isEnd = false;
        isPath = false;
        isBuildable = false;
        isTower = false;
        isNull = true;
    }

    public void setStart(){
        color = startC;

        setAllFalse();
        isStart = true;
    }

    public void setEnd(){
        color = endC;

        setAllFalse();
        isEnd = true;
    }

    public void setPath(){
        color = pathC;

        setAllFalse();
        isPath = true;
    }

    public void setBuildable(){
        color = buildableC;

        setAllFalse();
        isBuildable = true;
    }

    public void setTower(){
        color = buildableC;
        innerColor = towerC;

        setAllFalse();
        isTower = true;
    }

    public void setBasicTower(){
        isSlowTower = false;

        damage = damage * 2;
        if(damage == 0) {
            damage = 1;
        }
    }

    public void setSlowTower(){
        isSlowTower = true;

        damage = damage / 2;
    }

    public void setTowerReloading(){
        innerColor = reloadingC;

        isReloading = true;
    }

    public void setTowerReloaded(){
        innerColor = towerC;

        isReloading = false;
    }

    public void setNull(){
        color = nullC;

        setAllFalse();
        isNull = true;
    }

    public void upgrade(){
        towerCost = towerCost + towerUpgradeCost;

        ++towerType;

        //if(!isSlowTower) {
        switch (towerType) {
            case 2:
                towerC = Color.RED;
                //towerUpgradeCost = 8;
                break;
            case 3:
                towerC = Color.MAGENTA;
                //towerUpgradeCost = 16;
                break;
            case 4:
                towerC = Color.BLACK;
                break;
        }
        innerColor = towerC;
        //}

        damage = damage * 2;
        reload = defaultReload / towerType;

        findTowerCost();
    }

    public boolean isCollision(float x2, float y2){
        boolean collision = false;

        if(x2 > side[0] && x2 < side[2] && y2 > side[1] && y2 < side[3]){
            collision = true;
        }

        return collision;
    }

    public boolean isInRange(int x2, int y2){
        boolean inRange = false;

        if(x2 > side[0] - width && x2 < side[2] + width && y2 > side[1] - height && y2 < side[3] + height){
            inRange = true;
        }

        return inRange;
    }

    /*
    public boolean isReloading(){
        if(System.currentTimeMillis() - lastShot > reload){
            setTowerReloaded();
        } else {
            setTowerReloading();
        }
        return isReloading;
    }
    */

    public boolean isReloading(long pauseTime, long unPauseTime){
        if(lastShot > pauseTime) {
            if (System.currentTimeMillis() - lastShot > reload) {
                setTowerReloaded();
            } else {
                setTowerReloading();
            }
        }
        else if(unPauseTime > pauseTime){
            if ((System.currentTimeMillis() - (unPauseTime - pauseTime))- lastShot > reload) {
                setTowerReloaded();
            } else {
                setTowerReloading();
            }
        }
        return isReloading;
    }

    private void findTowerCost(){
        int nextTowerType = towerType + 1;
        towerUpgradeCost = (defaultTowerCost * (damage * 2 ) * nextTowerType) - towerCost;

        if(isSlowTower){
            towerUpgradeCost = towerUpgradeCost*2;
        }
    }
}



